/* giFTui
 * Copyright (C) 2003 the giFTui team
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "main.h"

#include "ui_search_cb.h"
#include "ui_search_meta.h"

/**/

#define SEARCH_MOTION_IDLE (500)

/**/

static gboolean
giftui_search_list_meta_popup (GiftuiSearch *se)
{
	gint x, y, width, height;
	gchar *filename, *icon;
	GtkTreeModel *model;
	
	if (!se->meta_tip)
	{
		if ((se->meta_tip = g_new (GiftuiSearchTip_t, 1)))
		{
			GtkWidget *hbox;
			
			se->meta_tip->window = gtk_window_new (GTK_WINDOW_POPUP);
			
			hbox = gtk_hbox_new (FALSE, 4);
			gtk_container_add (GTK_CONTAINER (se->meta_tip->window), hbox);
			gtk_container_set_border_width (GTK_CONTAINER (hbox), 4);
			
			se->meta_tip->icon = gtk_image_new ();
			gtk_box_pack_start (GTK_BOX (hbox), se->meta_tip->icon, FALSE, FALSE, 0);
			
			se->meta_tip->label = gtk_label_new (NULL);
			gtk_box_pack_start (GTK_BOX (hbox), se->meta_tip->label, FALSE, FALSE, 0);
			
			gtk_widget_show_all (hbox);
		}
		else
		{
			se->motion_idle = 0U;
			
			return FALSE;
		}
	}
	
	if (se->motion_in && se->motion_path)
	{
		GtkTreeIter iter;
		
		gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, NULL);
		
		model = gtk_tree_view_get_model (GTK_TREE_VIEW (se->list));
		
		if (gtk_tree_model_get_iter (model, &iter, se->motion_path))
		{
			GtkTreeIter parent;
			
			if (gtk_tree_model_iter_parent (model, &parent, &iter) &&
			    gtk_tree_model_iter_has_child (model, &iter))
			{
				gtk_tree_model_get (model, &iter, SEARCH_ICON, &icon,
						    SEARCH_FILENAME, &filename, -1);
				
				gtk_image_set_from_stock (GTK_IMAGE (se->meta_tip->icon),
							  icon, GTK_ICON_SIZE_BUTTON);
				gtk_label_set_text (GTK_LABEL (se->meta_tip->label),
						    filename);
			
				gtk_window_resize (GTK_WINDOW (se->meta_tip->window), 1, 1);
			
				gtk_window_get_size (GTK_WINDOW (se->meta_tip->window), &width, &height);
				gtk_window_move (GTK_WINDOW (se->meta_tip->window),
						 x - width / 2, y + 5);
				
				if (icon)
					g_free (icon);
				if (filename)
					g_free (filename);
				
				gtk_widget_show (se->meta_tip->window);
			}
		}
	}
	
	se->motion_idle = 0U;
	
	return FALSE;
}

gboolean
giftui_search_list_motion_timeout (GtkWidget *list,
				   GdkEventMotion *event,
				   GiftuiSearch *se)
{
	GtkTreeViewColumn *column;
	
	if (se->motion_path)
	{
		gtk_tree_path_free (se->motion_path);
		se->motion_path = NULL;
	}
	gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (list), event->x, event->y,
				       &se->motion_path, &column, NULL, NULL);
	if (se->motion_path)
		gtk_tree_view_get_cell_area (GTK_TREE_VIEW (list), se->motion_path,
					     column, &(se->motion_cell));
	
	/* Is the mouse pointer still in the same cell ? */
	if ((((gint) event->y) > se->motion_cell.y) &&
	    ((((gint) event->y) - se->motion_cell.height) < se->motion_cell.y) &&
	    (((gint) event->x) > se->motion_cell.x) &&
	    ((((gint) event->x) - se->motion_cell.width) < se->motion_cell.x))
		return FALSE;
	
	/* Else we must hide the popup window and wait the mouse stops
	 * to move. */
	if (se->meta_tip && GTK_WIDGET_VISIBLE (se->meta_tip->window))
		gtk_widget_hide (se->meta_tip->window);
	
	if (se->motion_idle)
		g_source_remove (se->motion_idle);
	
	se->motion_idle = g_timeout_add (SEARCH_MOTION_IDLE,
					 (GSourceFunc) giftui_search_list_meta_popup,
					 se);
	
	return FALSE;
}

gboolean
giftui_search_list_focus_inout (GtkWidget *widget,
				GdkEventProximity *event,
				GiftuiSearch *se)
{
	if (event->type == GDK_ENTER_NOTIFY)
		se->motion_in = TRUE;
	else
	{
		if (se->meta_tip)
			gtk_widget_hide (se->meta_tip->window);
		se->motion_in = FALSE;
	}
	
	return FALSE;
}
